#include <iostream>
#include <fstream>
#include <sstream>
#include "xml.h"

using namespace PA193XML;

/**
 * Print how to use this program.
 */
void usage() {
	std::cerr << "Wrong usage. 1 command line parameter is required." << std::endl;
	std::cerr << "Supply name of the file which contains XML data as a parameter." << std::endl;
}

/**
 * Open file from command line parameter. Print document if valid, print error code otherwise.
 * @return 0 on valid document, -1 on wrong arguments, positive integer on invalid document
 */
int main(int argc, char* argv[]) {
	if (argc != 2) {
		// supply at least one parameter
		usage();
		return -1;
	}
	std::string filename(argv[1]);

	std::ifstream input(filename);
	if (input.fail()) {
		std::cerr << "Cannot open file: " << filename << std::endl;
		return -1;
	}

	std::stringstream buffer;
	// copy everything from file to stringstream
	buffer << input.rdbuf();
	// close the file
	input.close();
	// extract the string from the stream
	std::string document_text = buffer.str();

	// create new parser
	Parser parser;
	// parse the string
	Document* document = parser.parse(&document_text);
	if (document == NULL) {
		std::cerr << "Document parsing failed: " << parser.get_error_code() << std::endl;
		return parser.get_error_code();
	}

	std::cout << "Document parsing passed:" << std::endl;
	std::cout << document->to_text() << std::endl;

	delete document;

	return 0;
}