/*
 ============================================================================
 Name        : iso2709.c
 Description : ISO 2709 parser header file
 ============================================================================
 */


/**
 * parse MARC21 in iso2709
 * @param in_file file stream to read from
 * @param out_file file stream to write output
 * @return 0 if no error occurs, error code otherwise;
 *
 */
int iso2709_parse_file(FILE* in_file, FILE* out_file);

/*
 * ERROR CODES
 * 1 - Error during reading from file
 * 2 - Record is too short to parse
 * 3 - Mismatch of given length and record length
 * 4 - Unable parse data offset
 * 5 - Invalid data offset position
 * 6 - Missing field definitions
 * 7 - Wrong number format
 * 8 - Mismatch in data fields
 * 9 - Writting error
 */


/*
 * init and free necessary structures
 */
int  _iso2709_init();
void _iso2709_free();

/**
 * checks whether string consists of numbers only
 */
int _iso2709_check_numbers(const char* in_str);

/**
 * reads next record from input file
 * @return length of record
 */
size_t _iso2709_read_next_record(FILE* in_file, char** outptr_bufffer);

/**
 * read data offset from record
 * @return data offset or 0
 */
size_t _iso2709_get_data_offset(const char* in_record, const size_t in_record_length);

/**
 * replace subfield indicators in string to '$'
 */
void _iso2709_replace_subfield_indicators(char* in_content, size_t length);

/**
 * parse given record of specified length and write output to out_file
 * @return 0 or error code
 */
int _iso2709_parse_record(const char* in_record, const size_t in_length, FILE* out_file);

/**
 * print record header to given stream, also used as check o writability of stream
 * @return 0 on success, 1 of failure`
 */
int _iso2709_print_header(const char* in_record, const size_t in_record_length, FILE* out_file);


