
#include <stdexcept>
#include <iostream>

#include <sexpressions.hpp>

using namespace std;

int main(int argc, char **argv) {
	if (argc != 1) {
		cerr << "Usage: " << argv[0] << endl;
		cerr << "The program uses standard input/output for data." << endl;
		return 1;
	}
	
	sexp::Parser *p = new sexp::Parser(cin);
	sexp::Writer *w = new sexp::CanonicalWriter(cout);
	
	
	while (!p->eof()) {
		// Read expression from stdin.
		sexp::Expression *e = NULL;
		try {
			e = p->parse_expression();
		}
		catch (std::runtime_error *err) {
			cerr << "Error while parsing s-expression file: " << err->what() << endl;
			return 1;
		}
		
		// Write expression in canonical form to stdout.
		w->write(e);
		cout << endl;
		
		delete e;
	}
	
	delete p;
	delete w;
	
	return 0;
}
