
#include <vector>
#include <iostream>

namespace sexp {
	
	struct Expression {
		bool is_octets;
		std::vector<uint8_t> display_hint;
		std::vector<uint8_t> octets;
		std::vector<Expression*> children;
	};
	
	class Writer {
	public:
		virtual void write(Expression *ex) = 0;
		virtual ~Writer() {
		}
	};
	
	class CanonicalWriter: public Writer {
		std::ostream *_output;
		void _write_octets(std::vector<uint8_t> &data);
	public:
		CanonicalWriter(std::ostream& out) {
			_output = &out;
		}
		
		virtual void write(Expression *ex) override;
	};
	
	class Parser {
		std::istream *_input;
		int _current;
		bool _pedantic;
		
		int _current_byte();
		void _advance();
		int _get_byte();
		void _skip_space();
		uint64_t _parse_decimal();
		Expression *_parse_string();
		void _parse_token(std::vector<uint8_t> &data);
		void _parse_string_raw(std::vector<uint8_t> &data);
		void _parse_verbatim(std::vector<uint8_t> &data, uint64_t len);
		void _parse_quoted(std::vector<uint8_t> &data);
		void _parse_hex(std::vector<uint8_t> &data);
		void _parse_base64(std::vector<uint8_t> &data);
		void _read_base64(std::vector<uint8_t> &data);
		Expression *_parse_list();
	public:
		Parser(std::istream& in): _input(&in) {
			_current = -1;
			_pedantic = true;
		}
	
		int line();
		int column();
		bool eof();
		
		Expression *parse_expression();
	};
}
